/**************************************************************************
 *  Nombre:  ibutton.h
 *  Version: 1.00
 *  Fecha:   30/5/07	
 *  Autor:   Ulises Bigliati
 *  Lenguaje:Ansi C
 *
 *  Descripcion
 * =======================================================================
 * El mdulo contine la declaracin de la funciones y constantes necesarias 
 * para leer iButtons 1990A de Dallas Semiconductors. 
 * La lectura se toma por defecto en el puerto A:
 * A0=Entrada A2=Salida . 
 * Pines 1 y 2 respectivamente en el conector X11 de la placa UNC20.
 * ========================================================================
*/
#ifndef IBUTTON_H_
#define IBUTTON_H_

#define IB_LECTURA_OK		1
#define IB_LECTURA_ERROR	0
#define IB_READY			1
#define IB_NOT_READY		0
/* *
 * Definir los pines que se utilizarn para IB_OUT e IB_IN 
 * Las constantes BSP_GPIO_PIN_Xn estn definidas en gpio.h
 * */
#define IB_OUT				BSP_GPIO_PIN_A2
#define IB_IN				BSP_GPIO_PIN_A0 

/* *********************************************************
 * Funcin ibReset()
 * *********************************************************
 * Descripcin:
 * 			Genera el ciclo de reset del bus 1-Wire e indica 
 * 			si hay un iButton listo para ser ledo en el bus.
 * Retorno: 
 * 			IB_READY (1)=> hay respuesta de un iButton
 *  		IB_NOT_READY (0)=> no hay iButtons para leer
 * *********************************************************/
unsigned int ibReset(void);
/* *********************************************************
 * Funcin ibRead()
 * *********************************************************
 * Descripcin:
 *			Una vez que ibReset() detecta un iButton en el bus
 * 			ibRead() debe ser llamada para obtener la lectura.
 * Retorno: 
 * 			IB_LECTURA_ERROR (0)=> fall la lectura
 *  		IB_LECTURA_OK (1)=> se obtuvo una lectura vlida
 * Argumentos:
 * 			Puntero a string de caractres de 8 bytes donde se
 * 			depositar la lectura del nmero de ID del iButton.
 * *********************************************************/
unsigned int ibRead(unsigned char * ns);


#endif /*IBUTTON_H_*/
