/*
 *            Copyright (c) 1998-2000 by NETsilicon Inc.
 *
 *  This software is copyrighted by and is the sole property of 
 *  NETsilicon.  All rights, title, ownership, or other interests
 *  in the software remain the property of NETsilicon.  This
 *  software may only be used in accordance with the corresponding
 *  license agreement.  Any unauthorized use, duplication, transmission,  
 *  distribution, or disclosure of this software is expressly forbidden.
 *
 *  This Copyright notice may not be removed or modified without prior
 *  written consent of NETsilicon.
 *
 *  NETsilicon, reserves the right to modify this software
 *  without notice.
 *
 *  NETsilicon
 *  411 Waverley Oaks Road                  USA 781.647.1234
 *  Suite 227                               http://www.netsilicon.com
 *  Waltham, MA 02452                       AmericaSales@netsilicon.com
 *
 *************************************************************************
 *
 *     Module Name: root.c
 *         Version: 1.00
 *   Original Date: 08/23/99	
 *          Author: Charles Gordon
 *        Language: Ansi C
 * Compile Options: 
 * Compile defines: 
 *       Libraries: 
 *    Link Options: 
 *
 *    Entry Points: applicationStart(), applicationTcpDown()
 *
 * Description.
 * =======================================================================
 * applicationStart() is the entry point for user applications. 
 * This function will be called after the kernel has started and the
 * TCP/IP stack has loaded.  applicationTcpDown() is called periodically
 * after the kernel has started while the system is waiting for the
 * TCP/IP stack to start.  
 *                                                                        
 *                                                                        
 * Edit Date/Ver   Edit Description
 * ==============  ===================================================
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <tx_api.h>
#include "bsp_api.h"
#include <appservices.h>

#include "appconf.h"
#include "gpio.h"
#include "ibutton.h" 

/*Declaracion de threads*/
TX_THREAD ib_thread ; //lectura de iButton
TX_THREAD pivot_thread ;//hilo de control de la aplicacin

/*Funcion de entrada a cada threads*/
void ib_thread_entry ();//iButton
void pivot_thread_entry (); //pivot

/***
 * Cola de mensajes utilizada para la comunicacion entre hilos 
 ***/
TX_QUEUE my_queue; 

/*
 *  Function: void applicationTcpDown (void)
 *
 *  Description:
 *
 *      This routine will be called by the NET+OS root thread once every 
 *      clock tick while it is waiting for the TCP/IP stack to come up.  
 *      This function can increment a counter everytime it's called to 
 *      keep track of how long we've been waiting for the stack to start.
 *      If we've been waiting too long, then this function can do something
 *      to handle the error.  
 *
 *      This function will not be called once the stack has started.
 *
 *  Parameters:
 *
 *      none
 *
 *  Return Values:
 *
 *      none
 *
 */

void applicationTcpDown (void){
    static int ticksPassed = 0; 
    ticksPassed++;  
/*
 * Code to handle error condition if the stack doesn't come up goes here. 
 */
}




/*
 *
 *  Function: void applicationStart (void)
 *
 *  Description:
 *
 *      This routine is responsible for starting the user application.  It should 
 *      create any threads or other resources the application needs.
 *
 *      ThreadX, the NET+OS device drivers, and the TCP/IP stack will be running
 *      when this function is called.
 *
 *  Parameters:
 *
 *      none
 *
 *  Return Values:
 *
 *      none
 *
 */
void applicationStart (void){
	/* Initialize the system services for the application. */
	initAppServices();

	/*Inicializacin del hardware: Se deben especificar los GPIO que conformarn
	 *el bus 1-Wire. Debe configurarse un pin de entrada y un pin de salida utilizando
	 *las constantes que figuran en GPIO.H
	 *En este caso utilizamo PORTA0 para entrada y PORTA2 para salida. 
	 */
	NAconfigureGPIOpin( BSP_GPIO_PIN_A0, NA_GPIO_INPUT_STATE, 1); //iButton IN
	NAconfigureGPIOpin( BSP_GPIO_PIN_A2, NA_GPIO_OUTPUT_STATE, 0);//iButton	OUT   
   
	/*
	 *  Crea una cola de intercomunicacin 128 bytes que empieza en la direccin devuelta por malloc()
	 * Un mensaje mide 2 words de 32-bit. (8 bytes)
	 * Si devuelve TX_SUCCESS, my_queue contiene lugar para 16 mensajes de 8 bytes c/u). 
	 */
	unsigned int status;
	status = tx_queue_create(&my_queue, "my_queue_name",TX_2_ULONG, malloc(128) , 128);   
	if (status==TX_SUCCESS) 
		printf("Cola de mensajes creada con xito\n");   
  	
    status = tx_thread_create ( &ib_thread, 
						"IButton Thread", 
						ib_thread_entry, 
						0,
						malloc (8192) ,
						8192,
						APP_DEFAULT_API_PRIORITY,
						APP_DEFAULT_API_PRIORITY,
						1 ,
						TX_AUTO_START);
	if (status==TX_SUCCESS)
		printf("Hilo de lectura de iButtons creado con xito\n");   
	
						
    status = tx_thread_create ( &pivot_thread, 
						"Pivot Thread", 
						pivot_thread_entry, 
						0,
	    				malloc (8192) ,
						8192,
						APP_DEFAULT_API_PRIORITY,
						APP_DEFAULT_API_PRIORITY,
						1,
						TX_AUTO_START);	   
	if (status==TX_SUCCESS)
		printf("Hilo de control creado con xito\n");  				
}

/* Punto de entrada del hilo de lectura de iButtons */
void ib_thread_entry (){
	unsigned char ns[8];
    unsigned long msg[2];
    int i;
    while(1){
		for(i=0;i<8;i++)
  				ns[i]=0;
		if (ibReset()==IB_READY){
			if ( ibRead(ns)==IB_LECTURA_OK ){	
   				//La lectura fue exitosa, entonces el hilo guarda el ID en 
   				//cola de mensajes y contina con su labor especfica.
   				
   				//Pone la lectura del iButton en la cola   				
   				memcpy(msg, ns, 8);
   				tx_queue_send(&my_queue, msg, TX_NO_WAIT);
				//Un retardo para eliminar mltiples lecturas	
				tx_thread_sleep(50);
			}
			else{
					// Lectura errnea
					// Hacer algo o simplemente descartar	 
			}
		}	
    }  
}

/* *
 * Punto de entrada del hilo de control de la aplicacin. En este caso
 * solo revisa la cola en busqueda de mensajes,  si encuentra uno, lo 
 * muestra por el standard output y contina. 
 * */
void pivot_thread_entry ()
{
	unsigned long msg[2];
	unsigned int status;	
	char ns[8];	
	int i;
	while(1){
			//Revisar la cola en busca de mensajes sin esperar(TX_NO_WAIT)
	 		status=tx_queue_receive(&my_queue, msg, TX_NO_WAIT );      
	    	
	    	//Si lo encuentra (TX_SUCCESS) es porque hay lecturas en cola,
	    	//entonces se muestra el ID por standard output 
	    	if (status==TX_SUCCESS){
						/*Obtiene el primer mensaje de la cola*/	
						memcpy(ns,msg, 8);
						//Lo muestra dandole el formato apropiado
						printf("Ibutton n ");
						for(i=0;i<8;i++)
							printf("%02X",ns[i]);				
						printf("\n");
			}		
	}
	return;
}

